"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.appendOrderLine = exports.addOrderContactData = exports.createOrderPayment = exports.postNewOrder = exports.updateCentrallyOrderExtraAttr = exports.pullOrdersForUser = exports.pullOrders = void 0;
const axios_1 = __importDefault(require("axios"));
const orderTypes_1 = require("./orderTypes");
const apiTools_1 = require("./apiTools");
/**
 Pulls open orders for venue. Uses provided access token to authenticate to rest api.
 * @param baseURL
 * @param {*} venue - we pull orders for this venue
 * @param {*} token - access token
 */
function pullOrders(baseURL, venue, token) {
    return __awaiter(this, void 0, void 0, function* () {
        console.log('Pulling orders...');
        const response = yield axios_1.default
            .get(`${baseURL}/ordering-api/api/orders/venue/${venue}`, {
            headers: {
                'Content-Type': 'application/json',
                Authorization: `Bearer ${token}`,
            },
        })
            .catch(apiTools_1.handleAPIError);
        if (!response)
            return [];
        const orders = [];
        for (const o of response.data) {
            if (o.completed) {
                orders.push(o);
            }
        }
        return orders;
    });
}
exports.pullOrders = pullOrders;
/**
 Pulls open orders for the user. Uses provided access token to authenticate to rest api.
 * @param baseURL
 * @param {*} token - access token
 * @param onlyCompletedOrders - option to pull also not yet completed orders
 */
function pullOrdersForUser(baseURL, token, onlyCompletedOrders = true) {
    return __awaiter(this, void 0, void 0, function* () {
        console.log('Pulling orders for user... only completed', onlyCompletedOrders);
        const response = yield axios_1.default
            .get(`${baseURL}/ordering-api/api/orders/opened`, {
            headers: {
                'Content-Type': 'application/json',
                Authorization: `Bearer ${token}`,
            },
        })
            .catch(apiTools_1.handleAPIError);
        if (!response)
            return [];
        const orders = [];
        for (const o of response.data) {
            if (!onlyCompletedOrders || o.completed) {
                orders.push(o);
            }
        }
        return orders;
    });
}
exports.pullOrdersForUser = pullOrdersForUser;
function updateCentrallyOrderExtraAttr(baseURL, token, orderId, store) {
    return __awaiter(this, void 0, void 0, function* () {
        let result;
        result = yield (0, axios_1.default)({
            method: 'POST',
            url: `${baseURL}/ordering-api/api/order/${orderId}/extra`,
            headers: {
                Authorization: `Bearer ${token}`,
                'Content-Type': 'application/json',
            },
            data: {
                store,
            },
        }).catch(apiTools_1.handleAPIError);
        return result ? true : false;
    });
}
exports.updateCentrallyOrderExtraAttr = updateCentrallyOrderExtraAttr;
function postNewOrder(baseURL, token, order) {
    return __awaiter(this, void 0, void 0, function* () {
        const response = yield (0, axios_1.default)({
            method: 'post',
            url: `${baseURL}/ordering-api/api/order/new`,
            headers: {
                'Content-Type': 'application/json',
                Authorization: `Bearer ${token}`,
            },
            data: order,
        }).catch(apiTools_1.handleAPIError);
        return response ? response.data : null;
    });
}
exports.postNewOrder = postNewOrder;
function createOrderPayment(baseURL, token, orderId, paymentType, amount, returnUrl, returnErrorUrl, source) {
    return __awaiter(this, void 0, void 0, function* () {
        const data = {
            orderId,
            paymentType,
            amount,
            returnUrl,
            returnErrorUrl,
            source,
        };
        const response = yield (0, axios_1.default)({
            method: 'post',
            url: `${baseURL}/payment-api/create`,
            headers: {
                'Content-Type': 'application/json',
                Authorization: `Bearer ${token}`,
            },
            data,
        }).catch(apiTools_1.handleAPIError);
        return response ? response.data : null;
    });
}
exports.createOrderPayment = createOrderPayment;
function addOrderContactData(baseURL, token, orderId, name, phone, email) {
    return __awaiter(this, void 0, void 0, function* () {
        const data = {
            name,
            phone,
            email,
        };
        const response = yield (0, axios_1.default)({
            method: 'post',
            url: `${baseURL}/ordering-api/api/order/${orderId}/contact`,
            headers: {
                'Content-Type': 'application/json',
                Authorization: `Bearer ${token}`,
            },
            data,
        }).catch(apiTools_1.handleAPIError);
        return response ? response.data : null;
    });
}
exports.addOrderContactData = addOrderContactData;
function appendOrderLine(baseURL, token, orderId, venueId, productId, price, quantity, status = orderTypes_1.EOrderLineStatus.NEW, productConfig, comment, extra) {
    return __awaiter(this, void 0, void 0, function* () {
        const data = {
            venue: venueId,
            lines: [
                {
                    productId,
                    price,
                    quantity,
                    status,
                    productConfig,
                    comment,
                    extra,
                },
            ],
        };
        const response = yield (0, axios_1.default)({
            method: 'post',
            url: `${baseURL}/ordering-api/api/order/${orderId}/append`,
            headers: {
                'Content-Type': 'application/json',
                Authorization: `Bearer ${token}`,
            },
            data,
        }).catch(apiTools_1.handleAPIError);
        return response ? response.data : null;
    });
}
exports.appendOrderLine = appendOrderLine;
