"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.connectWebSockets = void 0;
const StompJs = __importStar(require("@stomp/stompjs"));
const tools_1 = require("./tools");
if (typeof TextEncoder !== 'function') {
    const TextEncodingPolyfill = require('text-encoding');
    global.TextEncoder = TextEncodingPolyfill.TextEncoder;
    global.TextDecoder = TextEncodingPolyfill.TextDecoder;
}
/**
 * Create Stomp on websocket connection to Ordering Stack listening for new orders to process by handlers/callback (onMessageAsync)
 */
function connectWebSockets(params) {
    return __awaiter(this, void 0, void 0, function* () {
        const { appInsights, debugWs } = params.extra || {};
        let client;
        const baseURL = (0, tools_1.replaceProtocolInUrl)(params.baseURL, 'wss://') + '/websocket';
        let userUUID = '';
        const stompConfig = {
            brokerURL: baseURL,
            debug: function (a) {
                //console.log('DEBUG: ' + a);
            },
            reconnectDelay: 20000,
            heartbeatIncoming: 4000,
            heartbeatOutgoing: 4000,
            beforeConnect: function () {
                return __awaiter(this, void 0, void 0, function* () {
                    console.log('--- beforeConnect --- ');
                    debugWs &&
                        (appInsights === null || appInsights === void 0 ? void 0 : appInsights.trackTrace({ message: 'websocket beforeConnect' }));
                    let token = '';
                    let UUID = '';
                    let tries = 0;
                    while (!token && tries <= 8) {
                        const data = yield params
                            .authDataProvider();
                        token = data.token;
                        UUID = data.UUID;
                        // exponential backoff
                        !token && tries++;
                        !token && console.warn('WS no token. Exponential backoff');
                        !token &&
                            debugWs &&
                            (appInsights === null || appInsights === void 0 ? void 0 : appInsights.trackTrace({
                                message: 'websocket no token. Exponential backoff',
                            }));
                        !token &&
                            (yield new Promise((resolve) => setTimeout(resolve, 1000 * Math.pow(2, tries))));
                    }
                    if (!token) {
                        appInsights === null || appInsights === void 0 ? void 0 : appInsights.trackTrace({
                            message: 'websocket Access token provider error - deactivating socket',
                        });
                        console.error('Access token provider error - deactivating socket');
                        client.deactivate();
                        if (params.onAuthFailure)
                            params.onAuthFailure();
                    }
                    stompConfig.connectHeaders = { login: token /*passcode: ''*/ };
                    userUUID = UUID;
                });
            },
            onConnect: function () {
                return __awaiter(this, void 0, void 0, function* () {
                    if (!stompConfig.connectHeaders)
                        return;
                    const accessToken = stompConfig.connectHeaders.login;
                    yield params.onConnectedAsync(accessToken);
                    console.log('Websocket connected.');
                    if (params.onKDSMessageAsync) {
                        var subscription = client.subscribe(`/kds/${params.tenant}/${params.venue}`, function (data) {
                            return __awaiter(this, void 0, void 0, function* () {
                                var message = JSON.parse(data.body);
                                yield params.onKDSMessageAsync(message);
                            });
                        });
                    }
                    if (params.onOrdersUpdateAsync) {
                        var subscriptionForOrdersUpdate = client.subscribe(`/order-changes/${params.tenant}/${userUUID}`, function (data) {
                            return __awaiter(this, void 0, void 0, function* () {
                                var message = JSON.parse(data.body);
                                yield params.onOrdersUpdateAsync(message);
                            });
                        });
                    }
                    if (params.onNotificationAsync) {
                        var subscriptionForNotifications = client.subscribe(`/notifications/${params.tenant}/${userUUID}`, function (data) {
                            return __awaiter(this, void 0, void 0, function* () {
                                var message = JSON.parse(data.body);
                                yield params.onNotificationAsync(message);
                            });
                        });
                    }
                    if (params.onSteeringCommand && params.venue) {
                        client.subscribe(`/steering/${params.tenant}/${params.venue}`, function (data) {
                            var _a;
                            return __awaiter(this, void 0, void 0, function* () {
                                var message = JSON.parse(data.body);
                                (_a = params.onSteeringCommand) === null || _a === void 0 ? void 0 : _a.call(params, message);
                            });
                        });
                    }
                });
            },
            onDisconnect: function () {
                return __awaiter(this, void 0, void 0, function* () {
                    yield params.onDisconnectAsync();
                    console.log('Websocket disconnected.');
                });
            },
            onStompError: function (frame) {
                return __awaiter(this, void 0, void 0, function* () {
                    // Will be invoked in case of error encountered at Broker
                    // Bad login/passcode typically will cause an error
                    // Complaint brokers will set `message` header with a brief message. Body may contain details.
                    // Compliant brokers will terminate the connection after any error
                    console.error('onStompError headers.message: ' + (frame === null || frame === void 0 ? void 0 : frame.headers['message']));
                    debugWs &&
                        (appInsights === null || appInsights === void 0 ? void 0 : appInsights.trackTrace({
                            message: 'websocket onStompError headers.message: ' +
                                (frame === null || frame === void 0 ? void 0 : frame.headers['message']),
                        }));
                    console.error('onStompError body: ' + frame.body);
                    yield client.deactivate();
                    client = createNewClient(stompConfig);
                    client.activate();
                });
            },
            onWebSocketClose: function () {
                debugWs &&
                    (appInsights === null || appInsights === void 0 ? void 0 : appInsights.trackTrace({
                        message: 'websocket closed.',
                    }));
                console.log('Websocket closed.');
            },
            onWebSocketError: function (error) {
                debugWs &&
                    (appInsights === null || appInsights === void 0 ? void 0 : appInsights.trackTrace({
                        message: 'websocket error',
                        properties: { error },
                    }));
                console.log('Websocket error.');
            },
            onUnhandledMessage: function (message) {
                debugWs &&
                    (appInsights === null || appInsights === void 0 ? void 0 : appInsights.trackTrace({
                        message: 'websocket onUnhandledMessage',
                        properties: { message },
                    }));
            },
            logRawCommunication: true,
            discardWebsocketOnCommFailure: true,
        };
        try {
            // const { token, UUID } = await params
            //   .authDataProvider
            //   // params.baseURL,
            //   // params.refreshTokenHandler,
            //   // false,
            //   ();
            // stompConfig.connectHeaders = { login: token /*passcode: ''*/ };
            client = createNewClient(stompConfig);
            client.activate();
            return () => __awaiter(this, void 0, void 0, function* () {
                yield client.deactivate();
                console.log('Websocket connection deactivated');
            });
        }
        catch (err) {
            console.error(err);
            return () => Promise.resolve();
        }
    });
}
exports.connectWebSockets = connectWebSockets;
function createNewClient(stompConfig) {
    const _client = new StompJs.Client(stompConfig);
    if (typeof WebSocket !== 'function') {
        // Fallback code
        _client.webSocketFactory = () => {
            const WebSocket = require('ws');
            global.WebSocket = WebSocket;
            //console.log('create ws with ' + stompConfig.brokerURL);
            const ws = new WebSocket(stompConfig.brokerURL);
            return ws;
        };
    }
    return _client;
}
