var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import * as listener from './wsListener';
import * as orderService from './orderService';
import * as orderStore from './orderStore';
export function orderChangesListener(BASE_URL, TENANT, VENUE, authDataProvider, 
//refreshTokenHandler: IRefreshTokenStorageHandler,
onOrderUpdatedCallback, _onAuthFailureCallback, enableKDS, websocketMessageCallback, onSteeringCommand, extra) {
    const { appInsights, debugWs, onlyCompletedOrdersForUser } = extra || {};
    return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
        orderStore.setOrderStoreUpdatedCallback(onOrderUpdatedCallback);
        const params = {
            baseURL: BASE_URL,
            tenant: TENANT,
            venue: VENUE,
            authDataProvider: authDataProvider,
            onConnectedAsync: (accessToken) => __awaiter(this, void 0, void 0, function* () {
                console.log('Websocket connected async');
                debugWs &&
                    (appInsights === null || appInsights === void 0 ? void 0 : appInsights.trackTrace({ message: 'websocket connected async' }));
                let orders;
                if (!enableKDS) {
                    // pulls ONLY open orders
                    orders = yield orderService.pullOrdersForUser(BASE_URL, accessToken, onlyCompletedOrdersForUser);
                }
                else {
                    // KDS version
                    // pulls ONLY open orders
                    orders = yield orderService.pullOrders(BASE_URL, VENUE, accessToken);
                }
                orderStore.onUpdateAllOrders(orders, enableKDS, VENUE, appInsights);
                // for (const order of orders) {
                //   // this leads to hanging orders on KDS, if order was closed during websocket offline it will never we removed from store
                //   orderStore.onOrderUpdate(order, enableKDS, VENUE);
                // }
            }),
            onDisconnectAsync: () => __awaiter(this, void 0, void 0, function* () {
                console.log('websocket disconnected');
                debugWs &&
                    (appInsights === null || appInsights === void 0 ? void 0 : appInsights.trackTrace({ message: 'websocket disconnected' }));
            }),
            onKDSMessageAsync: null,
            onOrdersUpdateAsync: (order) => __awaiter(this, void 0, void 0, function* () {
                debugWs &&
                    (appInsights === null || appInsights === void 0 ? void 0 : appInsights.trackTrace({
                        message: 'websocket onOrdersUpdateAsync',
                        properties: { order },
                    }));
                //console.log(`new or updated order [${order.id}]`);
                orderStore.onOrderUpdate(order, enableKDS, VENUE);
            }),
            onNotificationAsync: (message) => __awaiter(this, void 0, void 0, function* () {
                debugWs &&
                    (appInsights === null || appInsights === void 0 ? void 0 : appInsights.trackTrace({
                        message: 'websocket onNotificationAsync',
                        properties: { message },
                    }));
                if (websocketMessageCallback) {
                    websocketMessageCallback(message);
                }
                orderStore.onOrderError(message);
            }),
            onAuthFailure: () => __awaiter(this, void 0, void 0, function* () {
                console.log('------------- MANUAL AUTH REQUIRED ----------------- ');
                debugWs &&
                    (appInsights === null || appInsights === void 0 ? void 0 : appInsights.trackTrace({
                        message: 'websocket onAuthFailure',
                    }));
                if (_onAuthFailureCallback)
                    yield _onAuthFailureCallback();
            }),
            enableKDS,
            onSteeringCommand: onSteeringCommand,
        };
        if (enableKDS) {
            params.onKDSMessageAsync = (message) => __awaiter(this, void 0, void 0, function* () {
                //console.log(`MESSAGE  order [${message.id}]`);
                orderStore.onOrderUpdate(message, enableKDS, VENUE);
            });
        }
        const disconnectFn = yield listener.connectWebSockets(params);
        resolve(disconnectFn);
    }));
}
