var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import axios from 'axios';
import { EOrderLineStatus } from './orderTypes';
import { handleAPIError } from './apiTools';
/**
 Pulls open orders for venue. Uses provided access token to authenticate to rest api.
 * @param baseURL
 * @param {*} venue - we pull orders for this venue
 * @param {*} token - access token
 */
export function pullOrders(baseURL, venue, token) {
    return __awaiter(this, void 0, void 0, function* () {
        console.log('Pulling orders...');
        const response = yield axios
            .get(`${baseURL}/ordering-api/api/orders/venue/${venue}`, {
            headers: {
                'Content-Type': 'application/json',
                Authorization: `Bearer ${token}`,
            },
        })
            .catch(handleAPIError);
        if (!response)
            return [];
        const orders = [];
        for (const o of response.data) {
            if (o.completed) {
                orders.push(o);
            }
        }
        return orders;
    });
}
/**
 Pulls open orders for the user. Uses provided access token to authenticate to rest api.
 * @param baseURL
 * @param {*} token - access token
 * @param onlyCompletedOrders - option to pull also not yet completed orders
 */
export function pullOrdersForUser(baseURL, token, onlyCompletedOrders = true) {
    return __awaiter(this, void 0, void 0, function* () {
        console.log('Pulling orders for user... only completed', onlyCompletedOrders);
        const response = yield axios
            .get(`${baseURL}/ordering-api/api/orders/opened`, {
            headers: {
                'Content-Type': 'application/json',
                Authorization: `Bearer ${token}`,
            },
        })
            .catch(handleAPIError);
        if (!response)
            return [];
        const orders = [];
        for (const o of response.data) {
            if (!onlyCompletedOrders || o.completed) {
                orders.push(o);
            }
        }
        return orders;
    });
}
export function updateCentrallyOrderExtraAttr(baseURL, token, orderId, store) {
    return __awaiter(this, void 0, void 0, function* () {
        let result;
        result = yield axios({
            method: 'POST',
            url: `${baseURL}/ordering-api/api/order/${orderId}/extra`,
            headers: {
                Authorization: `Bearer ${token}`,
                'Content-Type': 'application/json',
            },
            data: {
                store,
            },
        }).catch(handleAPIError);
        return result ? true : false;
    });
}
export function postNewOrder(baseURL, token, order) {
    return __awaiter(this, void 0, void 0, function* () {
        const response = yield axios({
            method: 'post',
            url: `${baseURL}/ordering-api/api/order/new`,
            headers: {
                'Content-Type': 'application/json',
                Authorization: `Bearer ${token}`,
            },
            data: order,
        }).catch(handleAPIError);
        return response ? response.data : null;
    });
}
export function createOrderPayment(baseURL, token, orderId, paymentType, amount, returnUrl, returnErrorUrl, source) {
    return __awaiter(this, void 0, void 0, function* () {
        const data = {
            orderId,
            paymentType,
            amount,
            returnUrl,
            returnErrorUrl,
            source,
        };
        const response = yield axios({
            method: 'post',
            url: `${baseURL}/payment-api/create`,
            headers: {
                'Content-Type': 'application/json',
                Authorization: `Bearer ${token}`,
            },
            data,
        }).catch(handleAPIError);
        return response ? response.data : null;
    });
}
export function addOrderContactData(baseURL, token, orderId, name, phone, email) {
    return __awaiter(this, void 0, void 0, function* () {
        const data = {
            name,
            phone,
            email,
        };
        const response = yield axios({
            method: 'post',
            url: `${baseURL}/ordering-api/api/order/${orderId}/contact`,
            headers: {
                'Content-Type': 'application/json',
                Authorization: `Bearer ${token}`,
            },
            data,
        }).catch(handleAPIError);
        return response ? response.data : null;
    });
}
export function appendOrderLine(baseURL, token, orderId, venueId, productId, price, quantity, status = EOrderLineStatus.NEW, productConfig, comment, extra) {
    return __awaiter(this, void 0, void 0, function* () {
        const data = {
            venue: venueId,
            lines: [
                {
                    productId,
                    price,
                    quantity,
                    status,
                    productConfig,
                    comment,
                    extra,
                },
            ],
        };
        const response = yield axios({
            method: 'post',
            url: `${baseURL}/ordering-api/api/order/${orderId}/append`,
            headers: {
                'Content-Type': 'application/json',
                Authorization: `Bearer ${token}`,
            },
            data,
        }).catch(handleAPIError);
        return response ? response.data : null;
    });
}
